#include <DallasTemperature.h>
#include <OneWire.h>
#include <LiquidCrystal_I2C.h>

#define ONE_WIRE_BUS 4 
#define LEDrelay 12
#define PUMPrelay 13
#define selectSwitch 5
#define pinB 1
#define pinA 0

const int       AirValue =            700;  // Calibrate Value by sensor in air
const int       WaterValue =          300;  // Calibrate Value by sensor in water
int             soilMoistureValue =   0;
int             soilmoisturepercent = 0;    
int             testCount =           0;
float           sysTEMP =             0.0;
volatile byte   aFlag =               0; 
volatile byte   bFlag =               0; 
int             encoderPos =          0; 
int             oldEncPos =           0; 
volatile byte   reading =             0; 
byte            rotarySwitch =        0;
int             UIstate =             0;
int             UIpos =               0;
byte            UInewstate =          0;
byte            change =              0;
byte            LEDstate =            1;
byte            AUTOstate =           0;

LiquidCrystal_I2C lcd = LiquidCrystal_I2C(0x27, 16, 2); // Change to (0x27,16,2) for 16x2 LCD.
OneWire oneWire(ONE_WIRE_BUS); 
DallasTemperature sensors(&oneWire);


void setup() {
  Serial.begin(9600); 
  pinMode(LEDrelay, OUTPUT);
  pinMode(PUMPrelay, OUTPUT);
  pinMode(pinA, INPUT_PULLUP);
  pinMode(pinB, INPUT_PULLUP);
  pinMode(selectSwitch, INPUT_PULLUP);
  attachInterrupt(0,PinA,RISING);
  attachInterrupt(1,PinB,RISING);
  lcd.init();
  lcd.backlight();
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("NASAPlanter V0.6");
  lcd.setCursor(1, 1);
  lcd.print("Jack Soldano" );
  digitalWrite(LEDrelay, HIGH);
  digitalWrite(PUMPrelay, HIGH);
  delay(2500);
}


void loop() {

  //Check Inputs
  soilMoistureValue = analogRead(A0);  //put Sensor insert into soil
    Serial.println(soilMoistureValue);
  soilmoisturepercent = map(soilMoistureValue, AirValue, WaterValue, 0, 100);
    Serial.print(soilmoisturepercent);
    Serial.println("%");
    Serial.println(soilMoistureValue);
  sensors.requestTemperatures(); // Send the command to get temperature readings 
    Serial.print("Temperature is: "); 
    Serial.print(sensors.getTempCByIndex(0));
  sysTEMP = sensors.getTempCByIndex(0);
  rotarySwitch = digitalRead(selectSwitch);

  //Check System Mode
  if (AUTOstate == 1){
    LEDstate = 0;
    //check moisture
    if (soilmoisturepercent >= 15 && soilmoisturepercent <= 40){
      Serial.println("Moisture Level correct in threshold "); 
    }
    // Water Satuation Warning
    else if (soilmoisturepercent > 41) {
      Serial.println("Water level too high!");   
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Water level too high");    
    }
    //Water Level Low
    else {
      Serial.println("Water level low!"); 
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Water Level Low! ");
      delay(1000);
      rotarySwitch = digitalRead(selectSwitch);
      if (rotarySwitch == 0){
        AUTOstate = 0;
        Serial.println("Exit auto mode!"); 
        lcd.clear();
        lcd.setCursor(0, 0);
        lcd.print("Exit auto mode! ");
        delay(1000);
      }
      if (AUTOstate == 1){
        WaterPumpSeq();
      }
      rotarySwitch = 1;
    }
    
  }
  if (LEDstate == 1){
    digitalWrite(LEDrelay, LOW);
  }

  if (UIstate == 0){
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Soil Temp: ");
    lcd.setCursor(11, 0);
    lcd.print(sysTEMP);
    lcd.setCursor(0, 1);
    lcd.print("Moisture: ");
    lcd.setCursor(10, 1);
    lcd.print(soilmoisturepercent);
    lcd.setCursor(13, 1);
    lcd.print("%");
    if (AUTOstate == 1){
      lcd.setCursor(15, 1);
      lcd.print("A");
    }
    else {
      lcd.setCursor(15, 1);
      lcd.print("M");
    }
    delay(500);

    /////////////////////////////////ENTER MENU SYSTEM /////////////////
    if (rotarySwitch != 1){
      UIstate = 1;
      UInewstate = 0;
      encoderPos = 0;
      rotarySwitch = 0;
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Entering Menu...");
      delay(1000);
    }
  }


  ////////////////////////////////Page 1 MENU SYSTEM /////////////////
  if (UIstate == 1 && UIpos == 0){  
    rotarySwitch = digitalRead(selectSwitch);  
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("1. Trigger Pump ");
    lcd.setCursor(6, 1);
    lcd.print("> 2");
    
    if (UIstate == 1 && UIpos == 0 && rotarySwitch != 1){  
      WaterPumpSeq();
    }
    
    if(oldEncPos != encoderPos) {
      if (encoderPos >> oldEncPos){
        UIpos++;
      }
      else{
        UIpos = 0;
      }
      oldEncPos = encoderPos;
      change = 1;
    }
   }
 

  ////////////////////////////////Page 2 MENU SYSTEM /////////////////
  else if (UIstate == 1 && UIpos == 1){    
    rotarySwitch = digitalRead(selectSwitch);  
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("2. Toggle Lights ");
    lcd.setCursor(3, 1);
    lcd.print("1 <  > 3");
    
    if (UIstate == 1 && UIpos == 1 && rotarySwitch != 1){  
      ToggleLights();
    }
    
    if(oldEncPos != encoderPos){
      if (encoderPos >> oldEncPos){
        UIpos++;
      }
      else{
        UIpos--;
      }
      oldEncPos = encoderPos;
    }
   }
  ////////////////////////////////Page 3 MENU SYSTEM /////////////////
  else if (UIstate == 1 && UIpos == 2){    
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("3. Auto Mode");
    lcd.setCursor(3, 1);
    lcd.print("2 < > 3");
    
    if (UIstate == 1 && UIpos == 2 && rotarySwitch != 1){ 
      ToggleAutoMode(); 
    }
    
    if(oldEncPos != encoderPos){
      if (encoderPos >> oldEncPos){
        UIpos++;
      }
      else{
        UIpos--;
      }
      oldEncPos = encoderPos;
    }
   }
   
  ////////////////////////////////Page 4 MENU SYSTEM /////////////////
  else if (UIstate == 1 && UIpos == 3){    
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("4. Exit Menu ");
    lcd.setCursor(3, 1);
    
    lcd.print("3 <");
    
    if (UIstate == 1 && UIpos == 3 && rotarySwitch != 1){  
      UIstate = 0;
      UIpos = 0;
    }
    
    if(oldEncPos != encoderPos){
      if (encoderPos >> oldEncPos){
        UIpos = 3;
      }
      else{
        UIpos--;
      }
      oldEncPos = encoderPos;
    }
   }
}

/////////////////////////////////////////////////////
///////////////////// FUNCTIONS /////////////////////
/////////////////////////////////////////////////////

void PinA(){
  Serial.println("PinA Int Triggered!"); 
  cli();
  reading = PIND & 0xC; 
  if(reading == B00001100 && aFlag) { 
    encoderPos --; 
    bFlag = 0; 
    aFlag = 0; 
  }
  else if (reading == B00000100) bFlag = 1; 
  sei(); //restart interrupts
}
 
void PinB(){
  Serial.println("PinB Int Triggered!"); 
  cli(); 
  reading = PIND & 0xC; 
  if (reading == B00001100 && bFlag) { 
    encoderPos ++; 
    bFlag = 0; 
    aFlag = 0; 
  }
  else if (reading == B00001000) aFlag = 1; 
  sei(); //restart interrupts
}

void SelectSwitch(){
  Serial.println("SelectSwitch Int Triggered!"); 
  cli(); 
  //rotarySwitch = 1;
  sei(); 
}

void WaterPumpSeq(){
  //stuff
  Serial.println("water pumpting stuff");
  digitalWrite(PUMPrelay, LOW);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Water Pumping");
  delay(1000);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Water Pumping.");
  delay(1000);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Water Pumping..");
  delay(1000);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Water Pumping...");
  delay(1000);
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Water Pump Fin.");
  digitalWrite(PUMPrelay, HIGH);
  delay(1000);
  
}

void ToggleLights(){
  //stuff
  Serial.println("water pumpting stuff");
  lcd.clear();
  lcd.setCursor(0, 0);
  lcd.print("Toggling Lights!");
  digitalWrite(LEDrelay, LOW);
  delay(1000);
  if (LEDstate == 1){
    LEDstate = 0;
    digitalWrite(LEDrelay, HIGH);
  }
  else{
    LEDstate = 1;
    digitalWrite(LEDrelay, LOW);
  }
}

void ToggleAutoMode(){
  Serial.println("Toggling Auto Farm Mode");
  if (AUTOstate == 0){
    AUTOstate = 1;
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Auto Mode On!");
    delay(1000);
    UIstate = 0;
    UIpos = 0;
  }
  else{
    AUTOstate = 0;
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Auto Mode OFF!");
    delay(1000);
    UIstate = 0;
    UIpos = 0;
  }
}
  
  
